#
# * prod_tcpdump
# *
# * Copyright (c) 2015 Ferrari electronic AG
#
"use strict"
_ = require("lodash")
path = require("path")
spawn = require("child_process").spawn
async = require 'async'
getmac = require 'getmac'

logger = require("../../../../common/devtool_nodelogger")
debug = require("debug")("omg-tcpdump")

Producer = require "../../ProducerClass"

class Tcpdump extends Producer
  isRunning = false
  tcpProcess = undefined
  tcpTimer = undefined

  cleanExit = ->
    if tcpProcess
      tcpProcess.kill()
    process.nextTick process.exit()
    return

  process.on "SIGINT", cleanExit # catch ctrl-c
  process.on "SIGTERM", cleanExit # catch kill
  process.on "exit", cleanExit

  constructor: ->
    super "tcpdump"

    @start = (device, duration) =>
      @debug "starting"
      unless duration
        logger.warn "You must specify a duration"
        @sendError "You must specify a duration"
        return

      #reset tcpTimer
      logger.debug "setting tcpdump timeout to " + duration + " minutes"
      if tcpTimer
        logger.debug "clearing tcpdump timeout"
        clearTimeout tcpTimer
      tcpTimer = setTimeout @stop, duration * 1000 * 60

      command = undefined
      filter = undefined
      args = undefined

      if tcpProcess and not tcpProcess.killed
        @sendError "tcpdump already running"
        return

      @sendStatus "started"

      filter = " not port 514 and not port 22"

      command = "tcpdump"

      # args = ["-a","-U","-i" + device, "-w /tmp/logs/capture.cap", "-p", "-n", "-s 0", filter,  " < /dev/null >/dev/null 2>/dev/null"];

      try
        fs.unlinkSync "/tmp/logs/capture.cap"
      catch e

      args = ["-a", "-U", "-i", "any", "-w", "/tmp/logs/capture.cap", "-p", "-n", "-s", "0", filter ]

      #spawn tcpdump
      logger.debug "running " + command + " " + args.join(" ")
      tcpProcess = spawn(command, args)
      @sendStatus "running"

      tcpProcess.stdout.on "data", (data) =>
        logger.debug "tcpdump stdout: " + data

      tcpProcess.stderr.on "data", (data) =>
        logger.debug "tcpdump stderr: " + data
        @sendData data.toString()
        # if data.toString().indexOf("listening on") > -1

      tcpProcess.on "exit", (err, code) =>
        @sendData code if code
        @sendError err if err
        @sendStatus "stopped"



    @stop = =>
      isRunning = false
      if tcpProcess and not tcpProcess.killed
        tcpProcess.once "error", (err) ->
          @sendError "tcpdump error: #{err.message}"
          return
        tcpProcess.kill()
      else
        logger.error "tcpProcess is not defined, or already killed"
        @sendStatus "stopped"

      #clear timer that kills processes after some time
      if tcpTimer
        logger.debug "clearing tcpdump timeout"
        clearTimeout tcpTimer

module.exports = new Tcpdump()